use bevy::prelude::*;

#[derive(Resource, Debug, Default)]
pub struct AssetLoader {
    pub images: Sprites,
    pub sounds: Sounds,
    pub fonts: Fonts,
}

#[derive(Resource, Debug, Default)]
pub struct Sprites {
    pub player: Handle<TextureAtlasLayout>,
    pub statue: Handle<TextureAtlasLayout>,
}

#[derive(Resource, Debug, Default)]
pub struct Sounds {
    pub meow: Handle<AudioSource>,
}

#[derive(Resource, Debug, Default)]
pub struct Fonts {
    pub iosevkalytemin: Handle<Font>,
}

pub struct AssetPlugin;

impl Plugin for AssetPlugin {
    fn build(&self, app: &mut App) {
        app.init_resource::<AssetLoader>()
            .add_systems(Startup, load_assets);
    }
}

fn load_assets(
    mut assets: ResMut<AssetLoader>,
    asset_server: Res<AssetServer>,
    mut texture_atlases: ResMut<Assets<TextureAtlasLayout>>,
) {
    let player_atlas = TextureAtlasLayout::from_grid(
        Vec2::new(32.0, 64.0),
        3,
        1,
        Some(Vec2 { x: 0., y: 0. }),
        Some(Vec2 { x: 0., y: 0. }),
    );
    let statue_atlas =
        TextureAtlasLayout::from_grid(Vec2::new(40., 74.), 1, 1, None, Some(Vec2::new(443., 20.)));

    let player_atlas_handle = texture_atlases.add(player_atlas);
    let statue_atlas_handle = texture_atlases.add(statue_atlas);

    *assets = AssetLoader {
        images: Sprites {
            player: player_atlas_handle,
            statue: statue_atlas_handle,
        },

        sounds: Sounds {
            meow: asset_server.load("sfx/meow.wav"),
        },

        fonts: Fonts {
            iosevkalytemin: asset_server.load("font/iosevka-lyteterm-regular.subset.ttf"),
        },
    };
}
